@@ -196,6 +196,24 @@ static const struct flag_uuid_mapping_t flag_uuid_mapping[] =
     { PED_PARTITION_SWAP,               PARTITION_SWAP_GUID },
 };
 
+static const efi_guid_t skip_set_system_guids[] =
+{
+    PARTITION_LVM_GUID,
+    PARTITION_SWAP_GUID,
+    PARTITION_RAID_GUID,
+    PARTITION_PREP_GUID,
+    PARTITION_SYSTEM_GUID,
+    PARTITION_BIOS_GRUB_GUID,
+    PARTITION_HPSERVICE_GUID,
+    PARTITION_MSFT_RESERVED_GUID,
+    PARTITION_BASIC_DATA_GUID,
+    PARTITION_MSFT_RECOVERY,
+    PARTITION_APPLE_TV_RECOVERY_GUID,
+    PARTITION_IRST_GUID,
+    PARTITION_CHROMEOS_KERNEL_GUID,
+    PARTITION_BLS_BOOT_GUID,
+};
+
 static const struct flag_uuid_mapping_t* _GL_ATTRIBUTE_CONST
 gpt_find_flag_uuid_mapping (PedPartitionFlag flag)
 {
@@ -234,7 +252,8 @@ struct __attribute__ ((packed)) _GuidPartitionEntryAttributes_t
   uint64_t NoBlockIOProtocol:1;
   uint64_t LegacyBIOSBootable:1;
   uint64_t Reserved:45;
-  uint64_t GuidSpecific:16;
+  uint64_t GuidSpecific:15;
+  uint64_t NoAutomount:1;
 #else
 #       warning "Using crippled partition entry type"
   uint32_t RequiredToFunction:1;
@@ -242,7 +261,8 @@ struct __attribute__ ((packed)) _GuidPartitionEntryAttributes_t
   uint32_t LegacyBIOSBootable:1;
   uint32_t Reserved:30;
   uint32_t LOST:5;
-  uint32_t GuidSpecific:16;
+  uint32_t GuidSpecific:15;
+  uint32_t NoAutomount:1;
 #endif
 };
 
@@ -1421,6 +1441,21 @@ gpt_partition_destroy (PedPartition *part)
   _ped_partition_free (part);
 }
 
+/* is_skip_guid checks the guid against the list of guids that should not be
+ * overridden by set_system. It returns a 1 if it is in the list.
+*/
+static bool
+is_skip_guid(efi_guid_t guid) {
+    int n = sizeof(skip_set_system_guids) / sizeof(skip_set_system_guids[0]);
+    for (int i = 0; i < n; ++i) {
+        if (guid_cmp(guid, skip_set_system_guids[i]) == 0) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 static int
 gpt_partition_set_system (PedPartition *part,
                           const PedFileSystemType *fs_type)
@@ -1431,6 +1466,11 @@ gpt_partition_set_system (PedPartition *part,
 
   part->fs_type = fs_type;
 
+  // Is this a GUID that should skip fs_type checking?
+  if (is_skip_guid(gpt_part_data->type)) {
+      return 1;
+  }
+
   if (fs_type)
     {
       if (strncmp (fs_type->name, "fat", 3) == 0
@@ -1538,6 +1578,24 @@ gpt_disk_is_flag_available(const PedDisk *disk, PedDiskFlag flag)
     }
 }
 
+static uint8_t*
+gpt_disk_get_uuid (const PedDisk *disk)
+{
+  GPTDiskData *gpt_disk_data = disk->disk_specific;
+
+  efi_guid_t uuid = gpt_disk_data->uuid;
+
+  /* uuid is always LE, while uint8_t is always kind of BE */
+
+  uuid.time_low = PED_SWAP32(uuid.time_low);
+  uuid.time_mid = PED_SWAP16(uuid.time_mid);
+  uuid.time_hi_and_version = PED_SWAP16(uuid.time_hi_and_version);
+
+  uint8_t *buf = ped_malloc(sizeof (uuid_t));
+  memcpy(buf, &uuid, sizeof (uuid_t));
+  return buf;
+}
+
 static int
 gpt_disk_get_flag (const PedDisk *disk, PedDiskFlag flag)
 {
@@ -1563,10 +1621,13 @@ gpt_partition_set_flag (PedPartition *part, PedPartitionFlag flag, int state)
   const struct flag_uuid_mapping_t* p = gpt_find_flag_uuid_mapping (flag);
   if (p)
   {
-    if (state)
+    if (state) {
       gpt_part_data->type = p->type_uuid;
-    else if (guid_cmp (gpt_part_data->type, p->type_uuid) == 0)
+    } else if (guid_cmp (gpt_part_data->type, p->type_uuid) == 0) {
+      // Clear the GUID so that fs_type will be used to return it to the default
+      gpt_part_data->type = PARTITION_LINUX_DATA_GUID;
       return gpt_partition_set_system (part, part->fs_type);
+    }
     return 1;
   }
 
@@ -1578,6 +1639,9 @@ gpt_partition_set_flag (PedPartition *part, PedPartitionFlag flag, int state)
     case PED_PARTITION_LEGACY_BOOT:
       gpt_part_data->attributes.LegacyBIOSBootable = state;
       return 1;
+    case PED_PARTITION_NO_AUTOMOUNT:
+      gpt_part_data->attributes.NoAutomount = state;
+      return 1;
     case PED_PARTITION_ROOT:
     case PED_PARTITION_LBA:
     default:
@@ -1603,6 +1667,8 @@ gpt_partition_get_flag (const PedPartition *part, PedPartitionFlag flag)
       return gpt_part_data->attributes.RequiredToFunction;
     case PED_PARTITION_LEGACY_BOOT:
       return gpt_part_data->attributes.LegacyBIOSBootable;
+    case PED_PARTITION_NO_AUTOMOUNT:
+      return gpt_part_data->attributes.NoAutomount;
     case PED_PARTITION_LBA:
     case PED_PARTITION_ROOT:
     default:
@@ -1622,6 +1688,7 @@ gpt_partition_is_flag_available (const PedPartition *part,
     {
     case PED_PARTITION_HIDDEN:
     case PED_PARTITION_LEGACY_BOOT:
+    case PED_PARTITION_NO_AUTOMOUNT:
       return 1;
     case PED_PARTITION_ROOT:
     case PED_PARTITION_LBA:
@@ -1686,6 +1753,61 @@ gpt_partition_get_name (const PedPartition *part)
   return gpt_part_data->translated_name;
 }
 
+
+static int
+gpt_partition_set_type_uuid (PedPartition *part, const uint8_t *uuid)
+{
+  GPTPartitionData *gpt_part_data = part->disk_specific;
+
+  efi_guid_t* type_uuid = &gpt_part_data->type;
+  memcpy(type_uuid, uuid, sizeof (efi_guid_t));
+
+  /* type_uuid is always LE, while uint8_t is always kind of BE */
+
+  type_uuid->time_low = PED_SWAP32(type_uuid->time_low);
+  type_uuid->time_mid = PED_SWAP16(type_uuid->time_mid);
+  type_uuid->time_hi_and_version = PED_SWAP16(type_uuid->time_hi_and_version);
+
+  return 1;
+}
+
+
+static uint8_t*
+gpt_partition_get_type_uuid (const PedPartition *part)
+{
+  const GPTPartitionData *gpt_part_data = part->disk_specific;
+
+  efi_guid_t type_uuid = gpt_part_data->type;
+
+  /* type_uuid is always LE, while uint8_t is always kind of BE */
+
+  type_uuid.time_low = PED_SWAP32(type_uuid.time_low);
+  type_uuid.time_mid = PED_SWAP16(type_uuid.time_mid);
+  type_uuid.time_hi_and_version = PED_SWAP16(type_uuid.time_hi_and_version);
+
+  uint8_t *buf = ped_malloc(sizeof (uuid_t));
+  memcpy(buf, &type_uuid, sizeof (uuid_t));
+  return buf;
+}
+
+static uint8_t*
+gpt_partition_get_uuid (const PedPartition *part)
+{
+  const GPTPartitionData *gpt_part_data = part->disk_specific;
+
+  efi_guid_t uuid = gpt_part_data->uuid;
+
+  /* uuid is always LE, while uint8_t is always kind of BE */
+
+  uuid.time_low = PED_SWAP32(uuid.time_low);
+  uuid.time_mid = PED_SWAP16(uuid.time_mid);
+  uuid.time_hi_and_version = PED_SWAP16(uuid.time_hi_and_version);
+
+  uint8_t *buf = ped_malloc(sizeof (uuid_t));
+  memcpy(buf, &uuid, sizeof (uuid_t));
+  return buf;
+}
+
 static int
 gpt_get_max_primary_partition_count (const PedDisk *disk)
 {
@@ -1781,9 +1903,15 @@ static PedDiskOps gpt_disk_ops =
 
   partition_set_name:		gpt_partition_set_name,
   partition_get_name:		gpt_partition_get_name,
+  partition_set_type_id:	NULL,
+  partition_get_type_id:	NULL,
+  partition_set_type_uuid:	gpt_partition_set_type_uuid,
+  partition_get_type_uuid:	gpt_partition_get_type_uuid,
+  partition_get_uuid:		gpt_partition_get_uuid,
   disk_set_flag:		gpt_disk_set_flag,
   disk_get_flag:		gpt_disk_get_flag,
   disk_is_flag_available:	gpt_disk_is_flag_available,
+  disk_get_uuid:		gpt_disk_get_uuid,
 
   PT_op_function_initializers (gpt)
 };
@@ -1793,7 +1921,8 @@ static PedDiskType gpt_disk_type =
   next:		NULL,
   name:		"gpt",
   ops:		&gpt_disk_ops,
-  features:	PED_DISK_TYPE_PARTITION_NAME
+  features:	PED_DISK_TYPE_PARTITION_NAME | PED_DISK_TYPE_PARTITION_TYPE_UUID |
+		PED_DISK_TYPE_DISK_UUID | PED_DISK_TYPE_PARTITION_UUID
 };
 
 void
