@@ -1,7 +1,7 @@
 #!/bin/sh
 # Probe Ext2, Ext3 and Ext4 file systems
 
-# Copyright (C) 2008-2014, 2019-2022 Free Software Foundation, Inc.
+# Copyright (C) 2008-2014, 2019-2023 Free Software Foundation, Inc.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,7 +42,8 @@ for type in ext2 ext3 ext4 btrfs xfs nilfs2 ntfs vfat hfsplus udf f2fs; do
   # create an $type file system, creation failures are not parted bugs,
   # skip the filesystem instead of failing the test.
   if [ "$type" = "xfs" ]; then
-      mkfs.xfs -ssize=$ss -dfile,name=$dev,size=${n_sectors}s || { warn_ "$ME: mkfs.$type failed, skipping"; continue; }
+      # XFS requires at least 300M which is > 1024 sectors with 8192b sector size
+      mkfs.xfs -ssize=$ss -dfile,name=$dev,size=300m || { warn_ "$ME: mkfs.$type failed, skipping"; continue; }
   else
       dd if=/dev/null of=$dev bs=$ss seek=$n_sectors >/dev/null || { warn_ "$ME: dd failed, skipping $type"; continue; }
       mkfs.$type $force $dev || { warn_ "$ME: mkfs.$type failed skipping"; continue; }
